<?php

declare(strict_types=1);

namespace App\Controller;

use App\Dto\Input\DeployImageInput;
use App\Entity\Command;
use App\Entity\Image;
use App\Entity\OrganizationalUnit;
use App\Model\DeployMethodTypes;
use App\Model\TraceStatus;
use App\Service\Trace\CreateService;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Contracts\HttpClient\HttpClientInterface;

class DeployImageAction extends AbstractController
{
    public function __construct(
        protected readonly EntityManagerInterface $entityManager,
        protected readonly HttpClientInterface    $httpClient,
        protected readonly CreateService          $createService,
        public readonly \App\Controller\OgAgent\DeployImageAction $deployImageOgAgentAction,
        public readonly \App\Controller\OgRepository\Image\DeployImageAction $deployImageOgRepositoryAction,
    )
    {
    }

    public function __invoke(DeployImageInput $input, Image $image): JsonResponse
    {
        $command = $this->entityManager->getRepository(Command::class)->findOneBy(['name' => 'Restaurar Imagen']);

        $partitionInfo = json_decode($image->getPartitionInfo(), true);

        $inputData = [
            'method' => $input->method,
            'client' => $input->client->getEntity()->getUuid(),
            'image' => $image->getUuid(),
            'p2pMode' => $input->p2pMode,
            'p2pTime' => $input->p2pTime,
            'mcastIp' => $input->mcastIp,
            'mcastPort' => $input->mcastPort,
            'mcastSpeed' => $input->mcastSpeed,
            'mcastMode' => $input->mcastMode,
            'numDisk' => (string) $partitionInfo['numDisk'],
            'numPartition' => (string) $partitionInfo['numPartition'],
        ];

        switch ($input->method){
            case DeployMethodTypes::UNICAST:
                $data = [
                    'dsk' => (string) $partitionInfo['numDisk'],
                    'par' => (string) $partitionInfo['numPartition'],
                    'ifs' => "1",
                    'cpt' => "83",
                    'idi' => $image->getUuid(),
                    'nci' => $image->getName(),
                    'ipr' => $image->getRepository()->getIp(),
                    'nfn' => 'RestaurarImagen',
                    'ptc' => DeployMethodTypes::UNICAST,
                    'ids' => '0'
                ];

                $agentJobId = $this->deployImageOgAgentAction->__invoke($image, $command, $input->client->getEntity());
                $this->createService->__invoke($input->client->getEntity(), $command, TraceStatus::IN_PROGRESS, $agentJobId, $inputData);

                break;

            case DeployMethodTypes::MULTICAST:
                $data = [
                    'dsk' => (string) $image->getPartitionInfo()['numDisk'],
                    'par' => (string) $image->getPartitionInfo()['numPartition'],
                    'cpt' => "83",
                    'idi' => $image->getUuid(),
                    'nci' => $image->getName(),
                    'ipr' => $image->getRepository()->getIp(),
                    'nfn' => 'CrearImagen',
                    'ids' => '0'
                ];

                $agentJobId = $this->deployImageOgAgentAction->__invoke($image, $command);
                $this->createService->__invoke($image->getClient(), $command, TraceStatus::IN_PROGRESS, $agentJobId, $inputData);

                break;
        }


        return new JsonResponse(data: [], status: Response::HTTP_OK);
    }
}
